@@ -8,7 +8,7 @@ use std::process::{Command, Stdio};
 use walkdir::WalkDir;
 
 const CRATE: &str = env!("CARGO_MANIFEST_DIR");
-const TEST_BINS_IN: &str = "tests/bin";
+const TEST_BINS_IN: &str = "tests/c-tests";
 
 fn find_files_with_extensions<'a>(
     exts: &'a [&'a str],
@@ -32,73 +32,21 @@ fn rerun_src(path: impl AsRef<Path>) {
         if let Some(path) = entry.to_str() {
             println!("cargo:rerun-if-changed={}", path)
         }
-    }
-}
-
-fn build_rs_tests(in_path: &Path, out_path: &Path) {
-    let filtered_env: HashMap<String, String> = std::env::vars()
-        .filter(|&(ref k, _)| {
-            k == "TERM"
-                || k == "TZ"
-                || k == "LANG"
-                || k == "PATH"
-                || k == "RUSTUP_HOME"
-                || k == "RUSTC_BOOTSTRAP"
-        })
-        .collect();
-
-    let target_name = "x86_64-unknown-linux-musl";
-
-    for in_source in find_files_with_extensions(&["rs"], &in_path) {
-        let stdout: Stdio = OpenOptions::new()
-            .write(true)
-            .open("/dev/tty")
-            .map(Stdio::from)
-            .unwrap_or_else(|_| Stdio::inherit());
-
-        let stderr: Stdio = OpenOptions::new()
-            .write(true)
-            .open("/dev/tty")
-            .map(Stdio::from)
-            .unwrap_or_else(|_| Stdio::inherit());
-
-        let output = in_source.file_stem().unwrap();
-
-        // Use the same toolchain channel of `rustc` as `cargo`, e.g. nightly
-        let cargo_path = PathBuf::from(std::env::var("CARGO").unwrap());
-        let cargo_path = cargo_path.parent().unwrap();
-        let rustc_path = cargo_path.join("rustc");
-        // Fallback to `rustc` in PATH, if it does not exist.
-        let rustc_path = if rustc_path.exists() {
-            rustc_path.as_os_str()
-        } else {
-            OsStr::new("rustc")
-        };
-
-        let status = Command::new(rustc_path)
-            .current_dir(&out_path)
-            .env_clear()
-            .envs(&filtered_env)
-            .stdout(stdout)
-            .stderr(stderr)
-            .arg("-C")
-            .arg("force-frame-pointers=yes")
-            .arg("-C")
-            .arg("debuginfo=2")
-            .arg("--target")
-            .arg(target_name)
-            .arg(&in_source)
-            .arg("-o")
-            .arg(output)
-            .status()
-            .unwrap_or_else(|_| panic!("failed to compile {:#?}", &in_source));
-
-        assert!(status.success(), "Failed to compile {:?}", &in_source);
+        if let Some(Some(path)) = entry.parent().map(Path::to_str) {
+            println!("cargo:rerun-if-changed={}", path)
+        }
     }
 }
 
 fn build_cc_tests(in_path: &Path, out_path: &Path) {
     for in_source in find_files_with_extensions(&["c", "s", "S"], &in_path) {
+        if let Some(path) = in_source.to_str() {
+            println!("cargo:rerun-if-changed={}", path)
+        }
+        if let Some(Some(path)) = in_source.parent().map(Path::to_str) {
+            println!("cargo:rerun-if-changed={}", path)
+        }
+
         let output = in_source.file_stem().unwrap();
 
         let mut cmd = cc::Build::new()
@@ -112,6 +60,7 @@ fn build_cc_tests(in_path: &Path, out_path: &Path) {
             .arg("-static-pie")
             .arg("-fPIC")
             .arg("-fno-omit-frame-pointer")
+            .arg("-fno-stack-protector")
             .arg("-g")
             .arg("-o")
             .arg(output)
@@ -123,18 +72,6 @@ fn build_cc_tests(in_path: &Path, out_path: &Path) {
     }
 }
 
-#[cfg(feature = "wasmldr")]
-fn build_wasm_tests(in_path: &Path, out_path: &Path) {
-    for wat in find_files_with_extensions(&["wat"], &in_path) {
-        let wasm = out_path
-            .join(wat.file_stem().unwrap())
-            .with_extension("wasm");
-        let bin = wat::parse_file(&wat).unwrap_or_else(|_| panic!("failed to compile {:?}", &wat));
-        std::fs::write(&wasm, &bin).unwrap_or_else(|_| panic!("failed to write {:?}", &wasm));
-        println!("cargo:rerun-if-changed={}", &wat.display());
-    }
-}
-
 // Build a binary named `bin_name` from the crate located at `in_dir`,
 // targeting `target_name`, then strip the resulting binary and place it
 // at `out_dir`/bin/`bin_name`.
@@ -151,22 +88,21 @@ fn cargo_build_bin(
 
     let filtered_env: HashMap<String, String> = std::env::vars()
         .filter(|&(ref k, _)| {
-            k == "TERM"
-                || k == "TZ"
-                || k == "LANG"
-                || k == "PATH"
-                || k == "RUSTUP_HOME"
-                || k == "RUSTC_BOOTSTRAP"
+            k == "TERM" || k == "TZ" || k == "LANG" || k == "PATH" || k == "RUSTUP_HOME"
         })
         .collect();
 
     let path = in_dir.as_os_str().to_str().unwrap();
 
     for p in [
+        "src",
+        "tests",
+        "build.rs",
         "Cargo.tml",
         "Cargo.toml",
         "Cargo.lock",
         "layout.ld",
+        ".cargo",
         ".cargo/config",
     ] {
         let file = in_dir.join(p);
@@ -191,7 +127,8 @@ fn cargo_build_bin(
         .map(Stdio::from)
         .unwrap_or_else(|_| Stdio::inherit());
 
-    let status = Command::new("cargo")
+    let mut cmd = Command::new("cargo");
+    let cmd = cmd
         .current_dir(&path)
         .env_clear()
         .envs(&filtered_env)
@@ -204,8 +141,15 @@ fn cargo_build_bin(
         .arg("--target")
         .arg(target_name)
         .arg("--bin")
-        .arg(bin_name)
-        .status()?;
+        .arg(bin_name);
+
+    #[cfg(feature = "gdb")]
+    let cmd = cmd.arg("--features=gdb");
+
+    #[cfg(feature = "dbg")]
+    let cmd = cmd.arg("--features=dbg");
+
+    let status = cmd.status()?;
 
     if !status.success() {
         eprintln!("Failed to build in {}", path);
@@ -273,9 +217,6 @@ fn main() {
     create(&out_dir_bin);
 
     build_cc_tests(&Path::new(CRATE).join(TEST_BINS_IN), &out_dir_bin);
-    build_rs_tests(&Path::new(CRATE).join(TEST_BINS_IN), &out_dir_bin);
-    #[cfg(feature = "wasmldr")]
-    build_wasm_tests(&Path::new(CRATE).join("tests/wasm"), &out_dir_bin);
 
     let target = "x86_64-unknown-linux-musl";
 
