@@ -1,7 +1,9 @@
 // SPDX-License-Identifier: Apache-2.0
 
-use log::{debug, info};
-use wasmtime_wasi::sync::WasiCtxBuilder;
+use crate::config;
+
+use log::debug;
+use wasmtime_wasi::sync::{TcpListener, WasiCtxBuilder};
 
 /// The error codes of workload execution.
 // clippy doesn't like how "ConfigurationError" ends with "Error", so..
@@ -32,6 +34,12 @@ impl From<std::io::Error> for Error {
     }
 }
 
+impl From<wasi_common::StringArrayError> for Error {
+    fn from(_err: wasi_common::StringArrayError) -> Self {
+        Self::StringTableError
+    }
+}
+
 impl From<wasmtime_wasi::Error> for Error {
     fn from(err: wasmtime_wasi::Error) -> Self {
         Self::WASIError(err)
@@ -73,11 +81,7 @@ pub type Result<T> = std::result::Result<T, Error>;
 // WasmFeatures, stdio handling, etc - but that gets messy quick.
 // Instead we should probably refactor this into distinct steps, each with
 // its own config options (and error variants - see above).
-pub fn run<T: AsRef<str>, U: AsRef<str>>(
-    bytes: impl AsRef<[u8]>,
-    args: impl IntoIterator<Item = T>,
-    envs: impl IntoIterator<Item = (U, U)>,
-) -> Result<Box<[wasmtime::Val]>> {
+pub fn run(bytes: impl AsRef<[u8]>, ldr_config: &config::Config) -> Result<Vec<wasmtime::Val>> {
     debug!("configuring wasmtime engine");
     let mut config = wasmtime::Config::new();
     // Support module-linking (https://github.com/webassembly/module-linking)
@@ -89,7 +93,7 @@ pub fn run<T: AsRef<str>, U: AsRef<str>>(
     config.static_memory_maximum_size(0);
     config.static_memory_guard_size(0);
     config.dynamic_memory_guard_size(0);
-    config.dynamic_memory_reserved_for_growth(0);
+    config.dynamic_memory_reserved_for_growth(16 * 1024 * 1024);
 
     let engine = wasmtime::Engine::new(&config).or(Err(Error::ConfigurationError))?;
 
@@ -103,19 +107,52 @@ pub fn run<T: AsRef<str>, U: AsRef<str>>(
 
     debug!("creating WASI context");
     let mut wasi = WasiCtxBuilder::new();
-    for arg in args {
-        wasi = wasi.arg(arg.as_ref()).or(Err(Error::StringTableError))?;
-    }
-    for kv in envs {
-        wasi = wasi
-            .env(kv.0.as_ref(), kv.1.as_ref())
-            .or(Err(Error::StringTableError))?;
+
+    debug!("Processing loader config {:#?}", &ldr_config);
+
+    if let Some(ref files) = ldr_config.files {
+        for file in files {
+            match (file.type_.as_ref(), file.name.as_ref()) {
+                ("stdio", "stdin") => wasi = wasi.inherit_stdin(),
+                ("stdio", "stdout") => wasi = wasi.inherit_stdout(),
+                ("stdio", "stderr") => wasi = wasi.inherit_stderr(),
+                _ => {}
+            }
+        }
     }
 
-    // v0.1.0 KEEP-CONFIG HACK: let wasmtime/wasi inherit our stdio.
-    // FIXME: this isn't a safe default if you don't trust the host!
-    info!("inheriting stdio from calling process");
-    wasi = wasi.inherit_stdio();
+    let mut num_fd = 3;
+    let mut fd_names: Vec<String> = Vec::new();
+
+    if let Some(ref files) = ldr_config.files {
+        for file in files {
+            match file.type_.as_ref() {
+                "tcp_listen" => {
+                    let port = file
+                        .port
+                        .expect("Config file `tcp_listen` has no `port` field set");
+                    let addr = file.addr.clone().unwrap_or_else(|| ":::".to_string());
+                    let stdlistener = std::net::TcpListener::bind((addr.as_str(), port))
+                        .unwrap_or_else(|e| panic!("Could not bind to {addr}:{port}: {e}"));
+                    stdlistener
+                        .set_nonblocking(true)
+                        .expect("Could not set nonblocking on TcpListener");
+
+                    wasi = wasi.preopened_socket(num_fd, TcpListener::from_std(stdlistener))?;
+                    num_fd += 1;
+                    wasi = wasi.env("LISTEN_FDS", &(num_fd - 3).to_string())?;
+                    fd_names.push(file.name.clone())
+                }
+                "stdio" => {}
+                field => {
+                    panic!("Unknown field '{field}' in config file");
+                }
+            }
+        }
+        if !fd_names.is_empty() {
+            wasi = wasi.env("LISTEN_FDNAMES", &fd_names.join(":"))?;
+        }
+    }
 
     debug!("creating wasmtime Store");
     let mut store = wasmtime::Store::new(&engine, wasi.build());
@@ -135,38 +172,27 @@ pub fn run<T: AsRef<str>, U: AsRef<str>>(
         .or(Err(Error::ExportNotFound))?;
 
     debug!("calling function");
-    func.call(store, Default::default())
-        .or(Err(Error::CallFailed))
+    let mut results = vec![wasmtime::Val::null(); func.ty(&store).results().len()];
+
+    func.call(store, Default::default(), &mut results)
+        .or(Err(Error::CallFailed))?;
+
+    Ok(results)
 }
 
 #[cfg(test)]
 pub(crate) mod test {
-    use crate::workload;
-    use std::iter::empty;
+    use crate::{config, workload};
 
-    const NO_EXPORT_WAT: &'static str = r#"(module
+    const NO_EXPORT_WAT: &str = r#"(module
       (memory (export "") 1)
     )"#;
 
-    const RETURN_1_WAT: &'static str = r#"(module
+    const RETURN_1_WAT: &str = r#"(module
       (func (export "") (result i32) i32.const 1)
     )"#;
 
-    const WASI_COUNT_ARGS_WAT: &'static str = r#"(module
-      (import "wasi_snapshot_preview1" "args_sizes_get"
-        (func $__wasi_args_sizes_get (param i32 i32) (result i32)))
-      (func (export "_start") (result i32)
-        (i32.store (i32.const 0) (i32.const 0))
-        (i32.store (i32.const 4) (i32.const 0))
-        (call $__wasi_args_sizes_get (i32.const 0) (i32.const 4))
-        drop
-        (i32.load (i32.const 0))
-      )
-      (memory 1)
-      (export "memory" (memory 0))
-    )"#;
-
-    const HELLO_WASI_WAT: &'static str = r#"(module
+    const HELLO_WASI_WAT: &str = r#"(module
       (import "wasi_snapshot_preview1" "proc_exit"
         (func $__wasi_proc_exit (param i32)))
       (import "wasi_snapshot_preview1" "fd_write"
@@ -196,12 +222,11 @@ pub(crate) mod test {
     fn workload_run_return_1() {
         let bytes = wat::parse_str(RETURN_1_WAT).expect("error parsing wat");
 
-        let results: Vec<i32> =
-            workload::run(&bytes, empty::<String>(), empty::<(String, String)>())
-                .unwrap()
-                .iter()
-                .map(|v| v.unwrap_i32())
-                .collect();
+        let results: Vec<i32> = workload::run(&bytes, &config::Config::default())
+            .unwrap()
+            .iter()
+            .map(wasmtime::Val::unwrap_i32)
+            .collect();
 
         assert_eq!(results, vec![1]);
     }
@@ -210,36 +235,16 @@ pub(crate) mod test {
     fn workload_run_no_export() {
         let bytes = wat::parse_str(NO_EXPORT_WAT).expect("error parsing wat");
 
-        match workload::run(&bytes, empty::<String>(), empty::<(String, String)>()) {
+        match workload::run(&bytes, &config::Config::default()) {
             Err(workload::Error::ExportNotFound) => {}
             _ => panic!("unexpected error"),
         };
     }
 
-    #[test]
-    fn workload_run_wasi_count_args() {
-        let bytes = wat::parse_str(WASI_COUNT_ARGS_WAT).expect("error parsing wat");
-
-        let results: Vec<i32> = workload::run(
-            &bytes,
-            vec!["a".to_string(), "b".to_string(), "c".to_string()],
-            vec![("k", "v")],
-        )
-        .unwrap()
-        .iter()
-        .map(|v| v.unwrap_i32())
-        .collect();
-
-        assert_eq!(results, vec![3]);
-    }
-
     #[test]
     fn workload_run_hello_wasi() {
         let bytes = wat::parse_str(HELLO_WASI_WAT).expect("error parsing wat");
-        let args: Vec<String> = vec![];
-        let envs: Vec<(String, String)> = vec![];
-
-        let results = workload::run(&bytes, args, envs).unwrap();
+        let results = workload::run(&bytes, &config::Config::default()).unwrap();
 
         assert_eq!(results.len(), 0);
 
