@@ -27,21 +27,27 @@
 //!
 //! Install the Webassembly rust toolchain:
 //!
-//!     $ rustup target install wasm32-wasi
+//! ```sh
+//! $ rustup target install wasm32-wasi
+//! ```
 //!
 //! Create simple rust program:
 //!
-//!     $ cargo init --bin hello-world
-//!     $ cd hello-world
-//!     $ echo 'fn main() { println!("Hello, Enarx!"); }' > src/main.rs
-//!     $ cargo build --release --target=wasm32-wasi
+//! ```sh
+//! $ cargo init --bin hello-world
+//! $ cd hello-world
+//! $ echo 'fn main() { println!("Hello, Enarx!"); }' > src/main.rs
+//! $ cargo build --release --target=wasm32-wasi
+//! ```
 //!
 //! Assuming you did install the `enarx` binary and have it in your `$PATH`, you can
 //! now run the Webassembly program in an Enarx keep.
 //!
-//!     $ enarx run target/wasm32-wasi/release/hello-world.wasm
-//!     […]
-//!     Hello, Enarx!
+//! ```sh
+//! $ enarx run target/wasm32-wasi/release/hello-world.wasm
+//! […]
+//! Hello, Enarx!
+//! ```
 //!
 //! If you want to suppress the debug output, add `2>/dev/null`.
 //!
@@ -51,17 +57,23 @@
 //! appropriate deployment backend. To see what backends are supported on your
 //! system, run:
 //!
-//!     $ enarx info
+//! ```sh
+//! $ enarx info
+//! ```
 //!
 //! You can manually select a backend with the `--backend` option, or by
 //! setting the `ENARX_BACKEND` environment variable:
 //!
-//!     $ enarx run --backend=sgx target/wasm32-wasi/release/hello-world.wasm
-//!     $ ENARX_BACKEND=sgx enarx run target/wasm32-wasi/release/hello-world.wasm
+//! ```sh
+//! $ enarx run --backend=sgx target/wasm32-wasi/release/hello-world.wasm
+//! $ ENARX_BACKEND=sgx enarx run target/wasm32-wasi/release/hello-world.wasm
+//! ```
 
 #![deny(clippy::all)]
 #![deny(missing_docs)]
-#![feature(asm)]
+#![warn(rust_2018_idioms)]
+// protobuf-codegen-pure would generate warnings
+#![allow(elided_lifetimes_in_paths)]
 
 mod backend;
 mod cli;
@@ -105,7 +117,13 @@ fn main() -> Result<()> {
         cli::Command::Exec(exec) => {
             let backend = exec.backend.pick()?;
             let binary = mmarinus::Kind::Private.load::<mmarinus::perms::Read, _>(&exec.binpath)?;
-            keep_exec(backend, backend.shim(), binary)
+            #[cfg(not(feature = "gdb"))]
+            let gdblisten = None;
+
+            #[cfg(feature = "gdb")]
+            let gdblisten = Some(exec.gdblisten);
+
+            keep_exec(backend, backend.shim(), binary, gdblisten)
         }
         cli::Command::Run(run) => {
             let modfile = File::open(run.module)?;
@@ -118,15 +136,43 @@ fn main() -> Result<()> {
             // then things will break mysteriously later on. So this assert
             // is just here to make them break earlier, and with less mystery.
             assert!(open_fd == 3, "module got unexpected fd {}", open_fd);
+
+            let configfile = match run.workldr.wasmcfgfile.as_ref() {
+                Some(name) => {
+                    let file = File::open(name)?;
+                    let fd = file.as_raw_fd();
+                    assert!(fd == 4, "config got unexpected fd {}", fd);
+
+                    Some(file)
+                }
+                None => None,
+            };
+
             // TODO: pass open_fd (or its contents) into the keep.
             let backend = run.backend.pick()?;
             let workldr = run.workldr.pick()?;
-            keep_exec(backend, backend.shim(), workldr.exec())
+            #[cfg(not(feature = "gdb"))]
+            let gdblisten = None;
+
+            #[cfg(feature = "gdb")]
+            let gdblisten = Some(run.gdblisten);
+
+            let res = keep_exec(backend, backend.shim(), workldr.exec(), gdblisten);
+            drop(configfile);
+            drop(modfile);
+            res
         }
+        #[cfg(feature = "backend-sev")]
+        cli::Command::Sev(cmd) => cli::sev::run(cmd),
     }
 }
 
-fn keep_exec(backend: &dyn Backend, shim: impl AsRef<[u8]>, exec: impl AsRef<[u8]>) -> Result<()> {
+fn keep_exec(
+    backend: &dyn Backend,
+    shim: impl AsRef<[u8]>,
+    exec: impl AsRef<[u8]>,
+    _gdblisten: Option<String>,
+) -> Result<()> {
     let keep = backend.keep(shim.as_ref(), exec.as_ref())?;
     let mut thread = keep.clone().spawn()?.unwrap();
     loop {
@@ -147,6 +193,11 @@ fn keep_exec(backend: &dyn Backend, shim: impl AsRef<[u8]>, exec: impl AsRef<[u8
                 block.msg.req.arg[3] = cpuid.edx.into();
             },
 
+            #[cfg(feature = "gdb")]
+            Command::Gdb(block, gdb_fd) => {
+                backend::handle_gdb(block, gdb_fd, _gdblisten.as_ref().unwrap());
+            }
+
             Command::Continue => (),
         }
     }
